package com.alivc.live.barestream_interactive;

import android.content.Context;
import android.widget.FrameLayout;

import com.alivc.live.commonbiz.LocalStreamReader;
import com.alivc.live.commonbiz.ResourcesConst;
import com.alivc.live.interactive_common.InteractLiveBaseManager;
import com.alivc.live.interactive_common.InteractiveMode;
import com.alivc.live.interactive_common.bean.InteractiveUserData;
import com.alivc.live.interactive_common.listener.InteractLivePushPullListener;
import com.alivc.live.interactive_common.utils.LivePushGlobalConfig;
import com.alivc.live.pusher.AlivcResolutionEnum;

import java.io.File;

class BareStreamController {

    private final InteractLiveBaseManager mInteractLiveManager;
    private final Context mContext;
    private final LocalStreamReader mLocalStreamReader;
    //主播预览 View
    private FrameLayout mAnchorRenderView;
    //观众连麦预览 View
    private FrameLayout mViewerRenderView;

    // 主播连麦推流信息
    private InteractiveUserData mPushUserData;
    // 观众连麦拉流信息
    private InteractiveUserData mPullUserData;

    public BareStreamController(Context context) {
        this.mContext = context;
        AlivcResolutionEnum resolution = LivePushGlobalConfig.mAlivcLivePushConfig.getResolution();
        int width = AlivcResolutionEnum.getResolutionWidth(resolution, LivePushGlobalConfig.mAlivcLivePushConfig.getLivePushMode());
        int height = AlivcResolutionEnum.getResolutionHeight(resolution, LivePushGlobalConfig.mAlivcLivePushConfig.getLivePushMode());
        mLocalStreamReader = new LocalStreamReader.Builder()
                .setVideoWith(width)
                .setVideoHeight(height)
                .setVideoStride(width)
                .setVideoSize(height * width * 3 / 2)
                .setVideoRotation(0)
                .setAudioSampleRate(44100)
                .setAudioChannel(1)
                .setAudioBufferSize(2048)
                .build();
        mInteractLiveManager = new InteractLiveBaseManager();
        mInteractLiveManager.init(context, InteractiveMode.BARE_STREAM);
    }

    /**
     * 设置主播预览 View
     *
     * @param frameLayout 主播预览 View
     */
    public void setAnchorRenderView(FrameLayout frameLayout) {
        this.mAnchorRenderView = frameLayout;
    }

    /**
     * 设置观众预览 View
     *
     * @param frameLayout 观众预览 View
     */
    public void setViewerRenderView(FrameLayout frameLayout) {
        this.mViewerRenderView = frameLayout;
    }

    /**
     * 开始直播
     */
    public void startPush(InteractiveUserData userData) {
        mPushUserData = userData;
        externAV();
        mInteractLiveManager.startPreviewAndPush(mPushUserData, mAnchorRenderView, true);
    }

    public void stopPreview() {
        mInteractLiveManager.stopPreview();
    }

    public void stopPush() {
        mPushUserData = null;
        mInteractLiveManager.stopPush();
    }

    public void stopCamera() {
        mInteractLiveManager.stopCamera();
    }

    private void externAV() {
        if (LivePushGlobalConfig.mAlivcLivePushConfig.isExternMainStream()) {
            File yuvFile = ResourcesConst.localCaptureYUVFilePath(mContext);
            mLocalStreamReader.readYUVData(yuvFile, (buffer, pts, videoWidth, videoHeight, videoStride, videoSize, videoRotation) -> {
                mInteractLiveManager.inputStreamVideoData(buffer, videoWidth, videoHeight, videoStride, videoSize, pts, videoRotation);
            });
            File pcmFile = ResourcesConst.localCapturePCMFilePath(mContext);
            mLocalStreamReader.readPCMData(pcmFile, (buffer, length, pts, audioSampleRate, audioChannel) -> {
                mInteractLiveManager.inputStreamAudioData(buffer, length, audioSampleRate, audioChannel, pts);
            });
        }
    }

    // 开始拉流
    public void startPull(InteractiveUserData userData) {
        mPullUserData = userData;
        mInteractLiveManager.setPullView(mPullUserData, mViewerRenderView, false);
        mInteractLiveManager.startPullRTCStream(mPullUserData);
    }

    // 结束拉流
    public void stopPull() {
        mInteractLiveManager.stopPullRTCStream(mPullUserData);
        mPullUserData = null;
    }

    /**
     * 主播是否正在连麦
     *
     * @return true:正在连麦  false:没有连麦
     */
    public boolean isPulling() {
        return mInteractLiveManager.isPulling(mPullUserData);
    }

    public void release() {
        mInteractLiveManager.release();
        mLocalStreamReader.stopYUV();
        mLocalStreamReader.stopPcm();
    }

    public void setInteractLivePushPullListener(InteractLivePushPullListener listener) {
        mInteractLiveManager.setInteractLivePushPullListener(listener);
    }

    public void switchCamera() {
        mInteractLiveManager.switchCamera();
    }

    public void enableSpeakerPhone(boolean enable) {
        mInteractLiveManager.enableSpeakerPhone(enable);
    }

    public void setMute(boolean b) {
        mInteractLiveManager.setMute(b);
    }

    public void enableAudioCapture(boolean enable) {
        mInteractLiveManager.enableAudioCapture(enable);
    }

    public void muteLocalCamera(boolean muteLocalCamera) {
        mInteractLiveManager.muteLocalCamera(muteLocalCamera);
    }

    public void enableLocalCamera(boolean enable) {
        mInteractLiveManager.enableLocalCamera(enable);
    }
}
